<?php
/**
*
* @package VC
* @version $Id$
* @copyright (c) 2009 phpBB Group
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* Table definitions
*/
global $table_prefix;
define('CAPTCHA_MATH_QUESTIONS_TABLE',	$table_prefix . 'captcha_math_questions');
define('CAPTCHA_MATH_CONFIRM_TABLE',	$table_prefix . 'math_confirm');

/**
* Placeholder for autoload
*/
if (!class_exists('phpbb_default_captcha'))
{
	include($phpbb_root_path . 'includes/captcha/plugins/captcha_abstract.' . $phpEx);
}

/**
* Math LaTeX captcha
*/
class phpbb_captcha_math extends phpbb_default_captcha
{
	var $captcha_vars = array(
		'captcha_math_mimetex'	=> 'CAPTCHA_MATH_MIMETEX',
	);
	
	var $answer;
	
	function phpbb_captcha_math()
	{
		if (!class_exists('captcha'))
		{
			global $phpbb_root_path, $phpEx;
			include($phpbb_root_path . 'includes/captcha/captcha_math.' . $phpEx);
		}
	}
	
	function init($type)
	{
		global $user;
		
		$this->seed = 0;
		$user->add_lang('captcha_math');
		
		parent::init($type);
	}
	
	/**
	* Returns an instance; does not have to be the same instance twice.
	*/
	function &get_instance()
	{
		$instance =& new phpbb_captcha_math();
		return $instance;
	}

	/**
	* See if the captcha has created its tables.
	*/
	function is_installed()
	{
		global $phpbb_root_path, $phpEx;
		global $db, $config;

		if (!class_exists('phpbb_db_tools'))
		{
			include($phpbb_root_path . 'includes/db/db_tools.' . $phpEx);
		}
		
		$db_tool = new phpbb_db_tools($db);
		
		return $db_tool->sql_table_exists(CAPTCHA_MATH_QUESTIONS_TABLE);
	}

	/**
	* Returns true if the captcha will work on the current install
	*/
	function is_available()
	{
		global $config, $db, $phpbb_root_path, $phpEx, $user;

		// required for dropdown language
		$user->add_lang('captcha_math');
		
		if (!phpbb_captcha_math::is_installed())
		{
			return false;
		}
		
		$sql = 'SELECT COUNT(question_id) as count
			FROM ' . CAPTCHA_MATH_QUESTIONS_TABLE; 
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);
		
		return ((bool) $row['count']);
	}
	
	/**
	* Returns the translated pretty name of the captcha.
	*/
	function get_name()
	{
		return 'CAPTCHA_MATH';
	}
	
	/**
	* Returns the class name of the captcha.
	*/
	function get_class_name()
	{
		return 'phpbb_captcha_math';
	}
	
	/**
	* Displays the configuration options in the ACP
	*/
	function acp_page($id, &$module)
	{
		global $user, $template, $db;
		global $config, $phpbb_root_path, $phpbb_admin_path, $phpEx;
		
		if (!$this->is_installed())
		{
			$this->install();
		}

		$user->add_lang('acp/board');
		$user->add_lang('captcha_math');

		$module->tpl_name = 'captcha_math_acp';
		$module->page_title = 'ACP_VC_SETTINGS';
		$form_key = 'acp_captcha';
		add_form_key($form_key);

		$submit = request_var('submit', '');
		$question_id = request_var('question_id', 0);
		$action = request_var('action', '');

		$list_url = $module->u_action . "&amp;configure=1&amp;select_captcha=" . $this->get_class_name();

		if ($action == 'edit' || $action == 'add')
		{
			$template->assign_vars(array(
				'UA_CAPTCHA_IMG'	=> str_replace('&amp;', '&', append_sid($phpbb_admin_path . 'index.' . $phpEx, 'captcha_demo=1&mode=visual&i=' . $id . '&select_captcha=' . $this->get_class_name())) . '&code=',
				'U_ACTION'		=> $list_url . "&amp;action=$action" . ($action == 'edit' ? "&amp;question_id=$question_id" : ''),
				'U_LIST'		=> $list_url,
			));

			$error = false;

			if ($submit)
			{
				$data = array(
					'question_text'		=> utf8_normalize_nfc(request_var('question_text', '', true)),
					'question_answer'	=> utf8_normalize_nfc(request_var('question_answer', '', true)),
				);

				if ($data['question_text'] === '')
				{
					$error = true;
				}

				if ($data['question_answer'] === '')
				{
					$error = true;
				}

				if (!$error)
				{
					if ($action == 'edit')
					{
						// update
						phpbb_captcha_math::update_question($question_id, $data);
					}
					else
					{
						// insert
						phpbb_captcha_math::add_question($data);
					}

					trigger_error($user->lang['QUESTION_STORED'] . adm_back_link($list_url));
				}
				
				$template->assign_vars(array(
					'QUESTION_TEXT'		=> $data['question_text'],
					'QUESTION_ANSWER'	=> $data['question_answer'],
				));
			}

			$template->assign_var('S_ERROR', $error);
		}

		if (($action == 'edit' || $action == 'delete') && $question_id)
		{
			$sql = 'SELECT question_id, question_text, question_answer
				FROM ' . CAPTCHA_MATH_QUESTIONS_TABLE . '
				WHERE question_id = ' . $question_id;
			$result = $db->sql_query_limit($sql, 1);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			if (!$row)
			{
				trigger_error($user->lang['FORM_INVALID'] . adm_back_link($list_url));
			}

			if ($action == 'delete')
			{
				phpbb_captcha_math::delete_question($question_id);
				trigger_error($user->lang['QUESTION_DELETED'] . adm_back_link($list_url));
			}
			else
			{
				// edit
				$template->assign_var('S_EDIT', true);

				if (!$submit)
				{
					$template->assign_vars(array(
						'QUESTION_TEXT'		=> $row['question_text'],
						'QUESTION_ANSWER'	=> $row['question_answer'],
					));
				}
			}
		}
		else if ($action == 'add')
		{
			// add
			$template->assign_var('S_ADD', true);
		}
		else
		{
			// config / list
			if ($submit && check_form_key($form_key))
			{
				if ($user->data['user_type'] != USER_FOUNDER)
				{
					trigger_error($user->lang['CONFIG_NO_PERMISSION'] . adm_back_link($module->u_action), E_USER_WARNING);
				}
				
				foreach (array_keys($this->captcha_vars) as $captcha_var)
				{
					$value = request_var($captcha_var, '');
					if ($value)
					{
						set_config($captcha_var, $value);
					}
				}
				trigger_error($user->lang['CONFIG_UPDATED'] . adm_back_link($list_url));
			}
			else if ($submit)
			{
				trigger_error($user->lang['FORM_INVALID'] . adm_back_link($list_url));
			}
			else
			{
				foreach ($this->captcha_vars as $captcha_var => $template_var)
				{
					$var = (isset($_REQUEST[$captcha_var])) ? request_var($captcha_var, '') : $config[$captcha_var];
					$template->assign_var($template_var, $var);
				}

				$template->assign_vars(array(
					'CAPTCHA_NAME'		=> $this->get_class_name(),
					'U_LIST'		=> $module->u_action,
					'U_ACTION'		=> $list_url,
					'S_DISPLAY_CONFIG'	=> $user->data['user_type'] == USER_FOUNDER,
				));
				
				$sql = 'SELECT question_id, question_text, question_answer
					FROM ' . CAPTCHA_MATH_QUESTIONS_TABLE . '
					ORDER BY question_text';
				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					$url = $list_url . "&amp;question_id={$row['question_id']}";

					$template->assign_block_vars('mathrow', array(
						'U_EDIT'	=> "{$url}&amp;action=edit",
						'U_DELETE'	=> "{$url}&amp;action=delete",
						'TEXT'		=> $row['question_text'],
						'ANSWER'	=> $row['question_answer'],

						'CONFIRM_IMAGE'		=> append_sid($phpbb_admin_path . 'index.' . $phpEx, 'captcha_demo=1&amp;mode=visual&amp;i=' . $id . '&amp;select_captcha=' . $this->get_class_name() . '&amp;code=' . rawurlencode(htmlspecialchars_decode($row['question_text']))),
						'CONFIRM_ID'		=> $this->confirm_id,
					));
				}
				$db->sql_freeresult($result);
			}
		}
	}
		
	/**
	* Delivers the demo image of image based captchas; not required for text/remote etc CAPTCHAs
	*/
	function execute_demo()
	{
		$captcha = new captcha();
		define('IMAGE_OUTPUT', 1);
		$captcha->execute(request_var('code', (string) $this->code), $this->seed);
	}
	
	/**
	* Returns the HTML needed to display a demo of the captcha
	*/
	function get_demo_template($id)
	{
		global $phpbb_admin_path, $phpEx;
		global $template;

		if (!$this->is_available())
		{
			return false;
		}
		
		$template->assign_vars(array(
			'CONFIRM_IMAGE'		=> append_sid($phpbb_admin_path . 'index.' . $phpEx, 'captcha_demo=1&amp;mode=visual&amp;i=' . $id . '&amp;select_captcha=' . $this->get_class_name()),
			'CONFIRM_ID'		=> $this->confirm_id,
		));

		return 'captcha_math_acp_demo.html';
	}

	function generate_code()
	{
		global $db, $user;
		
		$sql = 'SELECT question_id, question_text
			FROM ' . CAPTCHA_MATH_QUESTIONS_TABLE . '
			ORDER BY RAND()';
		$result = $db->sql_query_limit($sql, 1);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);
		
		$this->code = $row['question_text'];
		$this->confirm_id = md5(unique_id($user->ip));
		$this->solved = 0;

		$sql = 'INSERT INTO ' . CAPTCHA_MATH_CONFIRM_TABLE . ' ' . $db->sql_build_array('INSERT', array(
			'confirm_id'	=> (string) $this->confirm_id,
			'session_id'	=> (string) $user->session_id,
			'confirm_type'	=> (int) $this->type,
			'question_id'		=> (int) $row['question_id'],
		));
		$db->sql_query($sql);
	}

	function regenerate_code()
	{
		global $db, $user;
		
		// dirty hack: get rid of $this->confirm_code
		// needed so confirm_code is ignored on refresh
		$this->confirm_code = '';

		$sql = 'SELECT question_id, question_text
			FROM ' . CAPTCHA_MATH_QUESTIONS_TABLE . '
			ORDER BY RAND()';
		$result = $db->sql_query_limit($sql, 1);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);
		
		$this->code = $row['question_text'];
		$this->solved = 0;

		$sql = 'UPDATE ' . CAPTCHA_MATH_CONFIRM_TABLE . ' SET ' . $db->sql_build_array('UPDATE', array(
			'question_id'		=> (int) $row['question_id'],
		)) . ' WHERE confirm_id = \'' . $db->sql_escape($this->confirm_id) . '\'
			AND session_id = \'' . $db->sql_escape($user->session_id) . '\'';
		$db->sql_query($sql);
	}

	/**
	* New Question, if desired.
	*/
	function new_attempt()
	{
		global $db, $user;
		
		$sql = 'SELECT question_id, question_text
			FROM ' . CAPTCHA_MATH_QUESTIONS_TABLE . '
			ORDER BY RAND()';
		$result = $db->sql_query_limit($sql, 1);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);
		
		$this->code = $row['question_text'];
		$this->seed = 0;
		$this->solved = 0;

		$sql = 'UPDATE ' . CAPTCHA_MATH_CONFIRM_TABLE . ' SET ' . $db->sql_build_array('UPDATE', array(
			'question_id'			=> (int) $row['question_id'],
		)) . ', attempts = attempts + 1 
			WHERE confirm_id = \'' . $db->sql_escape($this->confirm_id) . '\' 
				AND session_id = \'' . $db->sql_escape($user->session_id) . '\'';
		$db->sql_query($sql);
	}
	
	/**
	* Look up everything we need for painting&checking.
	*/
	function load_code()
	{
		global $db, $user;

		$sql = 'SELECT q.question_text, q.question_answer, c.attempts
			FROM ' . CAPTCHA_MATH_CONFIRM_TABLE . ' c
			JOIN ' . CAPTCHA_MATH_QUESTIONS_TABLE . " q
				ON c.question_id = q.question_id
			WHERE confirm_id = '" . $db->sql_escape($this->confirm_id) . "'
				AND session_id = '" . $db->sql_escape($user->session_id) . "'
				AND confirm_type = " . $this->type;
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if ($row)
		{
			$this->code = $row['question_text'];
			$this->answer = $row['question_answer'];
			$this->attempts = $row['attempts'];
			return true;
		}

		return false;
	}

	function check_code()
	{
		return (strcasecmp(utf8_clean_string(str_replace(array(' ', "\n"), '', $this->answer)), utf8_clean_string(str_replace(array(' ', "\n"), '', $this->confirm_code))) === 0);
	}

	function delete_code()
	{
		global $db, $user;

		$sql = 'DELETE FROM ' . CAPTCHA_MATH_CONFIRM_TABLE . "
			WHERE confirm_id = '" . $db->sql_escape($confirm_id) . "'
				AND session_id = '" . $db->sql_escape($user->session_id) . "'
				AND confirm_type = " . $this->type;
		$db->sql_query($sql);
	}

	function reset()
	{
		global $db, $user;

		$sql = 'DELETE FROM ' . CAPTCHA_MATH_CONFIRM_TABLE . "
			WHERE session_id = '" . $db->sql_escape($user->session_id) . "'
				AND confirm_type = " . (int) $this->type;
		$db->sql_query($sql);

		// we leave the class usable by generating a new question
		$this->generate_code();
	}
	
	function get_template()
	{
		// in some cases false is returned
		if (!parent::get_template())
		{
			return false;
		}
		
		return 'captcha_math.html';
	}

	function garbage_collect($type)
	{
		global $db, $config;

		$sql = 'SELECT DISTINCT c.session_id
			FROM ' . CAPTCHA_MATH_CONFIRM_TABLE . ' c
			LEFT JOIN ' . SESSIONS_TABLE . ' s ON (c.session_id = s.session_id)
			WHERE s.session_id IS NULL' .
				((empty($type)) ? '' : ' AND c.confirm_type = ' . (int) $type);
		$result = $db->sql_query($sql);

		if ($row = $db->sql_fetchrow($result))
		{
			$sql_in = array();
			do
			{
				$sql_in[] = (string) $row['session_id'];
			}
			while ($row = $db->sql_fetchrow($result));

			if (sizeof($sql_in))
			{
				$sql = 'DELETE FROM ' . CAPTCHA_MATH_CONFIRM_TABLE . '
					WHERE ' . $db->sql_in_set('session_id', $sql_in);
				$db->sql_query($sql);
			}
		}
		$db->sql_freeresult($result);
	}

	function install()
	{
		global $config;
		global $db, $cache;
		
		if (!isset($config['captcha_math_mimetex']))
		{
			set_config('captcha_math_mimetex', '');
		}
		
		if (!class_exists('phpbb_db_tools'))
		{
			global $phpbb_root_path, $phpEx;
			include($phpbb_root_path . 'includes/db/db_tools.' . $phpEx);
		}
		
		$db_tool = new phpbb_db_tools($db);
		
		$schemas = array(
			CAPTCHA_MATH_QUESTIONS_TABLE	=> array (
				'COLUMNS' => array(
					'question_id'		=> array('UINT', NULL, 'auto_increment'),
					'question_text'		=> array('TEXT', ''),
					'question_answer'	=> array('XSTEXT_UNI', '', 'true_sort'),
				),
				'PRIMARY_KEY'			=> 'question_id',
			),
			CAPTCHA_MATH_CONFIRM_TABLE		=> array (
				'COLUMNS' => array(
					'session_id'		=> array('CHAR:32', ''),
					'confirm_id'		=> array('CHAR:32', ''),
					'question_id'		=> array('UINT', 0),
					'attempts'			=> array('UINT', 0),
					'confirm_type'		=> array('USINT', 0),
				),
				'KEYS'				=> array(
					'session_id'		=> array('INDEX', 'session_id'),
					'lookup'			=> array('INDEX', array('confirm_id', 'session_id')),
				),
				'PRIMARY_KEY'		=> 'confirm_id',
			),
		);
		
		foreach ($schemas as $table => $schema)
		{
			if (!$db_tool->sql_table_exists($table))
			{
				$db_tool->sql_create_table($table, $schema);
			}
		}
		
		$cache->purge();
	}

	function has_config()
	{
		return true;
	}

	function delete_question($question_id)
	{
		global $db;

		$sql = 'DELETE FROM ' . CAPTCHA_MATH_QUESTIONS_TABLE . '
			WHERE question_id = ' . (int) $question_id;
		$db->sql_query($sql);

		$sql = 'DELETE FROM ' . CAPTCHA_MATH_CONFIRM_TABLE . '
			WHERE question_id = ' . (int) $question_id;
		$db->sql_query($sql);
	}

	function add_question($data)
	{
		global $db;

		$sql = 'INSERT INTO ' . CAPTCHA_MATH_QUESTIONS_TABLE . ' ' . $db->sql_build_array('INSERT', $data);
		$db->sql_query($sql);
	}

	function update_question($question_id, $data)
	{
		global $db;

		$sql = 'UPDATE ' . CAPTCHA_MATH_QUESTIONS_TABLE . ' SET ' . $db->sql_build_array('UPDATE', $data) . '
			WHERE question_id = ' . (int) $question_id;
		$db->sql_query($sql);
	}
}
